///<reference path="../.d.ts"/>
"use strict";
var util = require("util");
var helpers = require("./helpers");
var yargs = require("yargs");
(function (OptionType) {
    OptionType[OptionType["String"] = 0] = "String";
    OptionType[OptionType["Boolean"] = 1] = "Boolean";
    OptionType[OptionType["Number"] = 2] = "Number";
    OptionType[OptionType["Array"] = 3] = "Array";
})(exports.OptionType || (exports.OptionType = {}));
var OptionType = exports.OptionType;
var OptionsBase = (function () {
    function OptionsBase(options, defaultProfileDir, $errors, $staticConfig) {
        var _this = this;
        this.options = options;
        this.defaultProfileDir = defaultProfileDir;
        this.$errors = $errors;
        this.$staticConfig = $staticConfig;
        this.optionsWhiteList = ["ui", "recursive", "reporter", "require", "timeout", "_", "$0"];
        _.extend(this.options, this.commonOptions);
        yargs.options(this.options);
        this.argv = yargs.argv;
        _.each(_.keys(this.argv), function (item) {
            _.each(_this.optionNames, function (optionName) {
                if (_this.options[optionName].alias === item) {
                    _this.argv[optionName] = _this.argv[item];
                }
            });
        });
        this.argv["profileDir"] = this.argv["profileDir"] || this.defaultProfileDir;
        _.each(this.optionNames, function (optionName) {
            Object.defineProperty(OptionsBase.prototype, optionName, {
                configurable: true,
                get: function () {
                    return this.getOptionValue(optionName);
                },
                set: function (value) {
                    this.argv[optionName] = value;
                }
            });
        });
    }
    Object.defineProperty(OptionsBase.prototype, "shorthands", {
        get: function () {
            var _this = this;
            var result = [];
            _.each(_.keys(this.options), function (optionName) {
                if (_this.options[optionName].alias) {
                    result.push(_this.options[optionName].alias);
                }
            });
            return result;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionsBase.prototype, "commonOptions", {
        get: function () {
            return {
                "log": { type: OptionType.String },
                "verbose": { type: OptionType.Boolean, alias: "v" },
                "path": { type: OptionType.String, alias: "p" },
                "version": { type: OptionType.Boolean },
                "help": { type: OptionType.Boolean, alias: "h" },
                "json": { type: OptionType.Boolean },
                "watch": { type: OptionType.Boolean },
                "avd": { type: OptionType.String },
                "profileDir": { type: OptionType.String },
                "timeout": { type: OptionType.String },
                "device": { type: OptionType.String },
                "availableDevices": { type: OptionType.Boolean },
                "appid": { type: OptionType.String },
                "geny": { type: OptionType.String },
                "debugBrk": { type: OptionType.Boolean },
                "debugPort": { type: OptionType.Number },
                "getPort": { type: OptionType.Boolean },
                "start": { type: OptionType.Boolean },
                "stop": { type: OptionType.Boolean },
                "ddi": { type: OptionType.String },
                "justlaunch": { type: OptionType.Boolean },
                "file": { type: OptionType.String },
                "force": { type: OptionType.Boolean, alias: "f" }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionsBase.prototype, "optionNames", {
        get: function () {
            return _.keys(this.options);
        },
        enumerable: true,
        configurable: true
    });
    OptionsBase.prototype.getOptionValue = function (optionName) {
        optionName = this.getCorrectOptionName(optionName);
        var result = typeof this.argv[optionName] === "number" ? this.argv[optionName].toString() : this.argv[optionName];
        var optionType = this.getOptionType(optionName);
        if (optionType === OptionType.Array && typeof result === "string") {
            return [result];
        }
        return result;
    };
    OptionsBase.prototype.validateOptions = function () {
        var _this = this;
        var parsed = Object.create(null);
        _.each(_.keys(this.argv), function (optionName) { return parsed[optionName] = _this.getOptionValue(optionName); });
        _.each(_.keys(parsed), function (originalOptionName) {
            var optionName = _this.getCorrectOptionName(originalOptionName);
            if (!_.contains(_this.optionsWhiteList, optionName)) {
                if (!_this.isOptionSupported(optionName)) {
                    _this.$errors.failWithoutHelp("The option '%s' is not supported. To see command's options, use '$ %s help %s'. To see all commands use '$ %s help'.", originalOptionName, _this.$staticConfig.CLIENT_NAME.toLowerCase(), process.argv[2], _this.$staticConfig.CLIENT_NAME.toLowerCase());
                }
                var optionType = _this.getOptionType(optionName);
                var optionValue = parsed[optionName];
                var parsedOptionType = typeof (optionValue);
                if (_.isArray(optionValue) && optionType !== OptionType.Array) {
                    _this.$errors.failWithoutHelp("You have set the %s option multiple times. Check the correct command syntax below and try again.", originalOptionName);
                }
                else if (_this.doesOptionRequireValue(optionType, parsedOptionType)) {
                    _this.$errors.failWithoutHelp("The option '%s' requires a value.", originalOptionName);
                }
                else if (optionType === OptionType.String && helpers.isNullOrWhitespace(optionValue)) {
                    _this.$errors.failWithoutHelp("The option '%s' requires non-empty value.", originalOptionName);
                }
                else if (optionType === OptionType.Boolean && parsedOptionType !== 'boolean') {
                    _this.$errors.failWithoutHelp("The option '%s' does not accept values.", originalOptionName);
                }
            }
        });
    };
    OptionsBase.prototype.getCorrectOptionName = function (optionName) {
        var secondaryOptionName = this.getSecondaryOptionName(optionName);
        return _.contains(this.optionNames, secondaryOptionName) ? secondaryOptionName : optionName;
    };
    OptionsBase.prototype.getOptionType = function (optionName) {
        var option = this.options[optionName] || this.tryGetOptionByAliasName(optionName);
        return option ? option.type : "";
    };
    OptionsBase.prototype.tryGetOptionByAliasName = function (aliasName) {
        var option = _.find(this.options, function (opt) { return opt.alias === aliasName; });
        return option;
    };
    OptionsBase.prototype.isOptionSupported = function (option) {
        if (!this.options[option]) {
            var opt = this.tryGetOptionByAliasName(option);
            return !!opt;
        }
        return true;
    };
    OptionsBase.prototype.doesOptionRequireValue = function (optionType, parsedOptionType) {
        return optionType !== OptionType.Boolean && parsedOptionType === 'boolean';
    };
    OptionsBase.prototype.getSecondaryOptionName = function (optionName) {
        var matchUpperCaseLetters = optionName.match(/(.+?)([-])([a-zA-Z])(.*)/);
        if (matchUpperCaseLetters) {
            var secondaryOptionName = util.format("%s%s%s", matchUpperCaseLetters[1], matchUpperCaseLetters[3].toUpperCase(), matchUpperCaseLetters[4] || '');
            return this.getSecondaryOptionName(secondaryOptionName);
        }
        return optionName;
    };
    return OptionsBase;
})();
exports.OptionsBase = OptionsBase;
